package com.example.demo.service;

import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.model.S3ObjectSummary;
import com.amazonaws.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class S3service {

    private final AmazonS3 s3;

    @Value("${source.bucket-name}")
    private String bucket;

    @Autowired
    public S3service(AmazonS3 amazonS3) {
        this.s3 = amazonS3;
    }

    public List<String> listFiles(String bkt) {
        // if method input is empty or null use default bucket else input
        String bucketName = StringUtils.isNullOrEmpty(bkt) ? bucket : bkt;

        return getKeys(bucketName).stream()
                .map(S3ObjectSummary::getKey)
                .collect(Collectors.toList());
    }

    public String moveAndDeleteKeyFromSource(String sKey, String dBucket) {

        String dKey = getDynamicKeyName(sKey);

        copyAndDeleteOp(sKey, dKey, dBucket);

        return "done";
    }

    public List<String> moveAllKeysToDestination(String dBucket) {
        List<String> names = new ArrayList<>();

        // get keys from source bucket

        for (S3ObjectSummary objectSummary : getKeys(bucket)) {

            String sKey = objectSummary.getKey();

            String dKey = getDynamicKeyName(sKey);

            copyAndDeleteOp(sKey, dKey, dBucket);

            names.add(sKey);
        }

        return names;
    }

    public String moveKeyToDestination(String sKey, String dKey, String dBucket) {

        // get keys from source bucket

        for (S3ObjectSummary objectSummary : getKeys(bucket)) {

            // filter given key

            if (objectSummary.getKey().equals(sKey)) {

                copyAndDeleteOp(sKey, dKey, dBucket);
            }
        }

        return "renamed.";
    }


    // private methods.

    private List<S3ObjectSummary> getKeys(String bucketName) {
        return s3.listObjectsV2(bucketName).getObjectSummaries();
    }

    private String getDynamicKeyName(String key) {
        return LocalDateTime.now() + "_" + key;
    }

    private void copyAndDeleteOp(String sKey, String dKey, String dBucket) {
        // copy key from source bucket from destination bucket
        s3.copyObject(bucket, sKey, dBucket, dKey);

        // delete object from source bucket
        s3.deleteObject(bucket, sKey);
    }
}
